<?php
class Plugin_entries extends Plugin {

  public function listing() {
    $app = Slim::getInstance();

    $folder      = $this->fetch_param('folder', null); // defaults to null
    $limit       = $this->fetch_param('limit', null, 'is_numeric'); // defaults to none
    $offset      = $this->fetch_param('offset', 0, 'is_numeric'); // defaults to zero
    $show_future = $this->fetch_param('show_future', false, false, true); // defaults to no
    $show_past   = $this->fetch_param('show_past', true, false, true); // defaults to yes
    $sort_by     = $this->fetch_param('sort_by', 'date'); // defaults to date
    $sort_dir    = $this->fetch_param('sort_dir', 'desc'); // defaults to desc
    $conditions  = $this->fetch_param('conditions', null); // defaults to null
    $slug        = $this->fetch_param('slug', null); // defaults to null
    $taxonomy    = $this->fetch_param('taxonomy', false, false, true); // defaults to false
    $switch      = $this->fetch_param('switch', null); // defaults to null

    if ($folder == null) {
      $folder = ltrim($this->fetch_param('from', $app->request()->getResourceUri()), "/");
    }

    if ($taxonomy) {
      list($type, $tax_slug) = Statamic::get_taxonomy_criteria($app->request()->getResourceUri());
      $tax_slug = urldecode($tax_slug);
      $conditions = "{$type}:{$tax_slug}";
    }

    if ($limit) {
      // override limit/offset if paging
      $app = Slim::getInstance();
      $pagination_variable = Statamic::get_pagination_variable();
      $page = $app->request()->get($pagination_variable) ? $app->request()->get($pagination_variable) : 1;
      $offset = ($page * $limit) - $limit;
    }

    if ($folder) {
      $list = Statamic::get_content_list($folder, $limit, $offset, $show_future, $show_past, $sort_by, $sort_dir, $conditions, $switch);
      if (sizeof($list)) {
        return $this->parse_loop($this->content, $list);
      } else {
        $arr = array();
        $arr['no_results'] = true;
        return $arr;
      }
    }
    return array();
  }


  public function pagination() {
    $folder         = $this->fetch_param('folder', null); // defaults to no
    $limit          = $this->fetch_param('limit', 10, 'is_numeric'); // defaults to none
    $show_future    = $this->fetch_param('show_future', false, false, true); // defaults to no
    $show_past      = $this->fetch_param('show_past', true, false, true); // defaults to yes
    $conditions     = $this->fetch_param('conditions', null); // defaults to null
    $taxonomy       = $this->fetch_param('taxonomy', false, false, true); // defaults to false

    $app = Slim::getInstance();
    if ($folder == null) {
      $folder = ltrim($this->fetch_param('from', $app->request()->getResourceUri()), "/");
    }

    if ($taxonomy) {
      list($type, $tax_slug) = Statamic::get_taxonomy_criteria($app->request()->getResourceUri());
      $tax_slug = urldecode($tax_slug);
      $conditions = "{$type}:{$tax_slug}";
    }

    $style = $this->fetch_param('style', 'prev_next'); // defaults to date
    $count = Statamic::get_content_count($folder, $show_future, $show_past, $conditions);

    $pagination_variable = Statamic::get_pagination_variable();
    $page = $app->request()->get($pagination_variable) ? $app->request()->get($pagination_variable) : 1;

    
    $arr = array();
    $arr['total_items']        = (int) max(0, $count);
    $arr['items_per_page']     = (int) max(1, $limit);
    $arr['total_pages']        = (int) ceil($count / $limit);
    $arr['current_page']       = (int) min(max(1, $page), max(1, $page));
    $arr['current_first_item'] = (int) min((($page - 1) * $limit) + 1, $count);
    $arr['current_last_item']  = (int) min($arr['current_first_item'] + $limit - 1, $count);
    $arr['previous_page']      = ($arr['current_page'] > 1) ? "?{$pagination_variable}=".($arr['current_page'] - 1) : FALSE;
    $arr['next_page']          = ($arr['current_page'] < $arr['total_pages']) ? "?{$pagination_variable}=".($arr['current_page'] + 1) : FALSE;
    $arr['first_page']         = ($arr['current_page'] === 1) ? FALSE : "?{$pagination_variable}=1";
    $arr['last_page']          = ($arr['current_page'] >= $arr['total_pages']) ? FALSE : "?{$pagination_variable}=".$arr['total_pages'];
    $arr['offset']             = (int) (($arr['current_page'] - 1) * $limit);

    return $this->parser->parse($this->content, $arr);
  }

}