<?php
class Statamic_Helper {

  static $date_pattern = "/\d{4}-\d{2}-\d{2}-/";

  public static function starts_with($haystack, $needle) {
      $length = strlen($needle);
      return (substr($haystack, 0, $length) === $needle);
  }

  public static function ends_with($haystack,$needle,$case=true) {
      if($case){return (strcmp(substr($haystack, strlen($haystack) - strlen($needle)),$needle)===0);}
      return (strcasecmp(substr($haystack, strlen($haystack) - strlen($needle)),$needle)===0);
  }


  public static function is_date_slug($slug) {
      if (preg_match(self::$date_pattern, $slug)) {
      $date_string = substr($slug, 0, 10);
      $date_array  = explode('-', $date_string);

      if (checkdate((int)$date_array[1], (int)$date_array[2], (int)$date_array[0]))
        return true;
    }

    return false;
  }

  public static function is_numeric_slug($slug) {
    if (preg_match("/\d\./i", $slug)) {
      return true;
    }
    return false;
  }

  public static function get_datestamp($date_slug) {
    $date_string = substr($date_slug, 0, 10);
    $date_array  = explode('-', $date_string);
    
    return strtotime($date_array[2].'-'.$date_array[1].'-'.$date_array[0]." 00:00");
  }

  public static function get_numeric($numeric_slug) {
    $numeric_string = substr($numeric_slug, 0, stripos($numeric_slug, "."));
    return $numeric_string;
  }

  # Strips double-slashes but preserves "http://"
  public static function reduce_double_slashes($string) {
    return preg_replace("#(^|[^:])//+#", "\\1/", $string);
  }


  public static function trim_slashes($string) {
    return trim($string, '/');
  }

  public static function remove_numerics_from_path($path) {
    if ($path) {
      $parts = explode("/", substr($path, 1));
      $fixedpath = "/";
      foreach ($parts as $part) {
        if (preg_match("/\d[\.\-]/i", $part)) {
          if (Statamic_helper::is_date_slug($part)) {
            $part  = preg_replace(self::$date_pattern, '', $part);
          } else {
            $part  = preg_replace("/\d*\./", '', $part);
          }
        }
        if ($fixedpath <> '/') $fixedpath .= '/';
        $fixedpath .= $part;
      }
      $path = $fixedpath;
    }
    return $path;
  }

  public static function pop_last_segment($url) {
    $url_array = explode('/', $url);
    
    array_pop($url_array);

    if (is_array($url_array))
      return implode('/', $url_array);

    return $url_array;
  }

  public static function resolve_path($path) {
    $content_root = Statamic::get_content_root();
    $content_type = Statamic::get_content_type();

    $parts = explode("/", substr($path, 1));
    $fixedpath = "/";
    foreach ($parts as $part) {
      if (file_exists("{$content_root}{$fixedpath}/{path}.{$content_type}") || is_dir("{$content_root}{$fixedpath}/{$part}")) {
        // don't rename it exists!
      } else {
        // check folders
        $list = Statamic::get_content_tree("{$fixedpath}", 1, 1);
        foreach ($list as $item) {
          if (Statamic_Helper::ends_with($item['slug'], $part)) {
            $part = $item['slug'];
            break;
          }
        }

        // check files
        $list = Statamic::get_file_list("{$fixedpath}");
        foreach ($list as $key => $item) {
          if (Statamic_Helper::ends_with($key, $part)) {
            $part = $key;
            break;
          }
        }
      }

      if ($fixedpath <> '/') $fixedpath .= '/';
      $fixedpath .= $part;
    }
    $path = $fixedpath;
    return $path;
  }

  public static function is_valid($uuid) {
    return preg_match('/^\{?[0-9a-f]{8}\-?[0-9a-f]{4}\-?[0-9a-f]{4}\-?[0-9a-f]{4}\-?[0-9a-f]{12}\}?$/i', $uuid) === 1;
  }

  public static function random_string($l = 32) {
    $s = '';
    $c = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxwz0123456789";
    for(;$l > 0;$l--) $s .= $c{rand(0,strlen($c)-1)};
    return str_shuffle($s);
  }

  public static function build_file_content($data_array, $content) {

    $file_content = "";
    $file_content .= Spyc::YAMLDump($data_array, false, 0);
    $file_content .= "---\n";
    $file_content .= $content;

    return $file_content;
  }
 }