<?php
class Statamic_View extends Slim_View
{
    static protected $_layout = NULL;
    static protected $_templates = NULL;
    static public $_dataStore = array();


    public function __construct() {
      Lex_Autoloader::register();
      $this->parser = new Lex_Parser();
      $this->parser->scope_glue(':');
      $this->parser->cumulative_noparse(true);
    }

    
    public static function set_templates($list) {
      self::$_templates = $list;
    }


    public static function set_layout($layout=NULL) {
      self::$_layout = $layout;
    }


    public function render($template) {
      $html = '<p style="text-align:center; font-size:28px; font-style:italic; padding-top:50px;">No template found.';
      if ($template) {
        $list = array($template);
      } else {
        $list = self::$_templates;
      }

      foreach ($list as $template) {
        $template_path = $this->getTemplatesDirectory() . '/templates/' . ltrim($template, '/');
        $template_type = 'html';
        if (file_exists($template_path.'.html') || file_exists($template_path.'.php')) {
          # standard lex-parsed template
          if (file_exists($template_path.'.html')) {
            
            Statamic_View::$_dataStore = array_merge(Statamic_View::$_dataStore, $this->data);
            $html = $this->parser->parse(file_get_contents($template_path.'.html'), Statamic_View::$_dataStore, array($this, 'callback'));

          # lets forge into raw date
          } elseif (file_exists($template_path.'.php')) {

            $template_type = 'php';

            extract($this->data);
            ob_start();
            require $template_path.".php";
            $html = ob_get_clean();
       
          } else {

            # you broke it.
            throw new RuntimeException('View cannot render template `' . $template_path . '`. Template does not exist.');
          }

          break;          
        }
      }
      return $this->_render_layout($html, $template_type);
    }

    
    public function _render_layout($_html, $template_type='html') {
        if (self::$_layout <> '') {

          $this->data['layout_content'] = $_html;
          $layout_path = $this->getTemplatesDirectory() . '/' . ltrim(self::$_layout, '/');

          if ($template_type == 'html') {

            if ( ! file_exists($layout_path.".html")) {
              return '<p style="text-align:center; font-size:28px; font-style:italic; padding-top:50px;">We can\'t find your theme files. Please check your settings.';
            }

            
            Statamic_View::$_dataStore = array_merge(Statamic_View::$_dataStore, $this->data);
            $html = $this->parser->parse(file_get_contents($layout_path.".html"), Statamic_View::$_dataStore, array($this, 'callback'), true);
            $html = Lex_Parser::inject_noparse($html);
           
          } else {

            extract($this->data);
            ob_start();
            require $layout_path.".php";
            $html = ob_get_clean();
          }

          return $html;

        }
        
        return $_html;
    }


    public static function callback($name, $attributes, $content) {
      Lex_Autoloader::register();
      $parser = new Lex_Parser();
      $parser->scope_glue(':');
      $parser->cumulative_noparse(true);

      $output = null;

      # single function plugins
      if (strpos($name, ':') === FALSE) {
        
        $plugin = $name;
        $call   = "index";

      } else {
        
        $pieces = explode(':', $name, 2);
        
        # no function exists
        if (count($pieces) != 2) return NULL;
          
        $plugin = $pieces[0];
        $call   = $pieces[1];
      }

      # check the plugin directories
      $plugin_folders = array('_add-ons/', '_app/statamic/core/');
      foreach ($plugin_folders as $folder) {

        if (is_dir($folder.$plugin) && is_file($folder.$plugin.'/pi.'.$plugin.'.php')) {
        
          $file = $folder.$plugin.'/pi.'.$plugin.'.php';
          break;
        
        } elseif (is_file($folder.'/pi.'.$plugin.'.php')) {
        
          $file = $folder.'/pi.'.$plugin.'.php';
          break;
        }
      }

      # plugin exists
      if (isset($file)) {

        require_once($file);
        $class = 'Plugin_'.$plugin;

        #formatted properly
        if (class_exists($class)) {
          $plug = new $class();
        }

        $output = false;

        # function exists
        if (method_exists($plug, $call)) {
          $plug->attributes = $attributes;
          $plug->content    = $content;

          $output = $plug->$call();
        } elseif (class_exists($class) && ! method_exists($plug, $call)) {
          $output = $class::$call();
        } 
          
        if (is_array($output)) {
          $output = $parser->parse($content, $output, array('Statamic_View', 'callback'));
        }
      }
      return $output;
      
    }
}